import os
import json
import re

from obspy.core import read as read_ms
from obspy.io.reftek.core import Reftek130Exception

from sohstationviewer.model.mseed.mseed import MSeed
from sohstationviewer.model.reftek.reftek import RT130
from sohstationviewer.database.extractData import signatureChannels
from sohstationviewer.controller.util import validateFile, displayTrackingInfo


def loadData(dataType, parent, listOfDir, reqWFChans=[], reqSOHChans=[],
             readStart=None, readEnd=None):
    """
    Go through root dir and read all files in that dir and its subdirs
    """
    print("dataType:", dataType)
    dataObject = None
    for d in listOfDir:
        if dataObject is None:
            if dataType == 'RT130':
                dataObject = RT130(
                    parent, d, reqWFChans=reqWFChans, reqSOHChans=reqSOHChans,
                    readStart=readStart, readEnd=readEnd)
                print("dataObject:", dataObject)
            else:
                try:
                    dataObject = MSeed(
                        parent, d, reqWFChans=reqWFChans,
                        reqSOHChans=reqSOHChans,
                        readStart=readStart, readEnd=readEnd)
                except Exception as e:
                    msg = f"Dir {d} can't be read due to error: {str(e)}"
                    displayTrackingInfo(parent, msg, "Warning")
                    pass
            # if dataObject.hasData():
            #     continue
            # If no data can be read from the first dir, throw exception
            # raise Exception("No data can be read from ", d)
        # TODO: will work with select more than one dir later
        # else:
        #     dataObject.readDir(d)

    # return dataObject.plottingData
    return dataObject


def readChannels(parent, listOfDir):
    """
    TODO: have to solve this later for changes of data type
    Scan available channels for channel prefer dialog
    """
    dataObject = None
    for d in listOfDir:
        if dataObject is None:
            # dataObject = Reftek.Reftek(parent, d)
            # if dataObject.hasData():
            #     continue

            # dataObject = MSeed_Text(parent, d, readChanOnly=True)
            dataObject = MSeed(parent, d, readChanOnly=True)
            if len(dataObject.channels) == 0:
                # If no data can be read from the first dir, throw exception
                raise Exception("No data can be read from ", d)
        else:
            dataObject.readDir(d, readChanOnly=True)
    return dataObject.channels


def detectDataType(parent, listOfDir):
    """
    For each dir in listOfDir,  use getDataTypeFromFile to identify the type
    of data for that dir if find a signature channel
    return:
        + None if there are more than one types of data detected
        + dataType found, Unknown data maybe return
    """
    sign_chan_dataType_dict = signatureChannels()

    dirDataTypeDict = {}
    for d in listOfDir:
        dataType = "Unknown"
        for path, subdirs, files in os.walk(d):
            for fileName in files:
                path2file = os.path.join(path, fileName)
                if not validateFile(path2file, fileName):
                    continue
                ret = getDataTypeFromFile(path2file, sign_chan_dataType_dict)
                if ret is not None:
                    dataType, chan = ret
                    break
            if dataType != "Unknown":
                break
        if dataType == "Unknown":
            dirDataTypeDict[d] = ("Unknown", '_')
        else:
            dirDataTypeDict[d] = (dataType, chan)
    dataTypeList = {d for d in dirDataTypeDict.values()}
    if len(dataTypeList) > 1:
        dirDataTypeStr = json.dumps(dirDataTypeDict)
        dirDataTypeStr = re.sub(r'\{|\}|"', '', dirDataTypeStr)
        dirDataTypeStr = re.sub(r', ', '\n', dirDataTypeStr)
        msg = (f"There are more than one types of data detected:\n"
               f"{dirDataTypeStr}\n\n"
               f"Please have only data that related to each other.")
        displayTrackingInfo(parent, msg, "error")
        return
    # elif list(dirDataTypeDict.values())[0] == "Can't be identified.":
    #     msg = (f"There are no known data detected.\n"
    #            f"Please select different folder(s).")
    #     displayTrackingInfo(parent, msg, "error")
    #     return
    return list(dirDataTypeDict.values())[0]


def getDataTypeFromFile(path2file, sign_chan_dataType_dict):
    stream = None
    try:
        stream = read_ms(path2file)
    except TypeError:
        return
    except Reftek130Exception:
        return 'RT130', '_'

    for trace in stream:
        chan = trace.stats['channel']
        if chan in sign_chan_dataType_dict.keys():
            return sign_chan_dataType_dict[chan], chan
