
from sohstationviewer.database.proccessDB import executeDB_dict, executeDB
from sohstationviewer.conf.dbSettings import dbConf


def getChanPlotInfo(orgChan, dataType):
    """
    Given chanID read from raw data file and detected dataType
    Return plotting info from DB for that channel
    """
    chan = orgChan
    if orgChan.startswith('EX'):
        chan = 'EX?'
    if orgChan.startswith('VM'):
        chan = 'VM?'
    if orgChan.startswith('MP'):
        chan = 'MP?'
    if orgChan.startswith('Event DS'):
        chan = 'Event DS?'
    if orgChan.startswith('DS'):
        chan = 'DS?'
    if orgChan.startswith('Disk Usage'):
        chan = 'Disk Usage?'
    if dbConf['seisRE'].match(chan):
        chan = 'SEISMIC'

    o_sql = ("SELECT channel, plotType, height, unit, linkedChan,"
             " convertFactor, label, fixPoint, valueColors "
             "FROM Channels as C, Parameters as P")
    if dataType == 'Unknown':
        sql = f"{o_sql} WHERE channel='{chan}' and C.param=P.param"
    else:
        sql = (f"{o_sql} WHERE channel='{chan}' and C.param=P.param"
               f" and dataType='{dataType}'")
    # print("SQL:", sql)
    chanInfo = executeDB_dict(sql)

    if len(chanInfo) == 0:
        chanInfo = executeDB_dict(
            f"{o_sql} WHERE channel='DEFAULT' and C.param=P.param")
    else:
        if chanInfo[0]['channel'] == 'SEISMIC':
            chanInfo[0]['label'] = dbConf['seisLabel'][orgChan[-1]]
        chanInfo[0]['channel'] = orgChan

    chanInfo[0]['label'] = (
        '' if chanInfo[0]['label'] is None else chanInfo[0]['label'])
    chanInfo[0]['unit'] = (
        '' if chanInfo[0]['unit'] is None else chanInfo[0]['unit'])
    chanInfo[0]['fixPoint'] = (
        0 if chanInfo[0]['fixPoint'] is None else chanInfo[0]['fixPoint'])
    if chanInfo[0]['label'].strip() == '':
        chanInfo[0]['label'] = chanInfo[0]['channel']
    else:
        chanInfo[0]['label'] = '-'.join([chanInfo[0]['channel'],
                                         chanInfo[0]['label']])
    if chanInfo[0]['label'].strip() == 'DEFAULT':
        chanInfo[0]['label'] = 'DEFAULT-' + orgChan
    return chanInfo[0]


def getWFPlotInfo(orgChan):
    chanInfo = executeDB_dict(
        "SELECT * FROM Parameters WHERE param='Seismic data'")
    chanInfo[0]['label'] = getChanLabel(orgChan)
    chanInfo[0]['unit'] = ''
    chanInfo[0]['channel'] = 'SEISMIC'
    return chanInfo[0]


def getChanLabel(chanID):
    if chanID.startswith("DS"):
        label = chanID
    else:
        label = chanID + '-' + dbConf['seisLabel'][chanID[-1]]
    return label


def signatureChannels():
    """
    return the dict {channel: dataType} in which channel is unique for dataType
    """
    sql = ("SELECT channel, dataType FROM Channels where channel in"
           "(SELECT channel FROM Channels GROUP BY channel"
           " HAVING COUNT(channel)=1)")
    rows = executeDB_dict(sql)
    sign_chan_dataType_dict = {r['channel']: r['dataType'] for r in rows}
    return sign_chan_dataType_dict


def getColorDef():
    sql = "SELECT color FROM TPS_ColorDefinition ORDER BY name ASC"
    rows = executeDB(sql)
    return [r[0] for r in rows]


def getColorRanges():
    sql = "SELECT name, baseCounts FROM TPS_ColorRange"
    rows = executeDB(sql)
    rangeNames = [r[0] for r in rows]
    baseCounts = [r[1] for r in rows]
    allSquareCounts = []
    clrLabels = []
    cnt = 0
    # 7 : number of color definition, not include 'E'
    for idx, bc in enumerate(baseCounts):
        allSquareCounts.append([0] * 7)
        clrLabels.append(['0 counts'])
        for cidx in range(1, 7):
            cnt = (bc * 10 ** (cidx - 1))
            allSquareCounts[idx][cidx] = cnt ** 2
            clrLabels[idx].append("+/- {:,} counts".format(cnt))
        clrLabels[idx].append("> {:,} counts".format(cnt))
    return rangeNames, allSquareCounts, clrLabels
