import os

from tempfile import mkdtemp
import shutil

from sohstationviewer.controller.util import displayTrackingInfo
from sohstationviewer.conf import constants


class WrongDataTypeError(Exception):
    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)


class DataTypeModel():
    def __init__(self, trackingBox, folder, readChanOnly=False,
                 reqWFChans=[], reqSOHChans=[],
                 readStart=0, readEnd=constants.HIGHEST_INT,
                 *args, **kwargs):
        self.trackingBox = trackingBox
        self.dir = folder
        self.reqSOHChans = reqSOHChans
        self.reqWFChans = reqWFChans
        self.readChanOnly = readChanOnly

        self.readStart = readStart      # start time to read in epoch
        self.readEnd = readEnd          # end time to read in epoch
        self.processingLog = []     # [(message, type)]
        self.logData = {'TEXT': []}     # 'TEXT': for text only file
        self.waveformData = {}
        self.SOHData = {}
        self.massPosData = {}
        self.dataTime = {}       # (earliestepoch,latestepoch) for each station

        self.channels = set()
        self.station = None
        self.nets = set()
        self.stats = set()
        self.gaps = {}
        self.timeRanges = {}
        self.selectedStaID = None
        # # channel with the longest traces total for each station
        # self.maxTraceTotalChan = {}
        self.tmpDir = mkdtemp()
        # print("currentdir:", os.getcwd())
        # tmpDirName = 'datatmp'
        # self.tmpDir = os.path.join(os.getcwd(), tmpDirName)
        try:
            os.mkdir(self.tmpDir)
        except FileExistsError:
            shutil.rmtree(self.tmpDir)
            os.mkdir(self.tmpDir)

        # self.readDir(folder, readChanOnly)

    def __del__(self):
        print("delete dataType Object")
        try:
            shutil.rmtree(self.tmpDir)
        except OSError as e:
            self.trackInfo(
                "Error deleting %s : %s" % (self.tmpDir, e.strerror),
                "error")
            print("Error deleting %s : %s" % (self.tmpDir, e.strerror))
        print("finish deleting")

    def hasData(self):
        if (len(self.logData) == 0 and len(self.SOHData) == 0 and
                len(self.massPosData) == 0 and len(self.waveformData) == 0):
            return False
        return True

    def trackInfo(self, text, type):
        displayTrackingInfo(self.trackingBox, text, type)
        if type != 'info':
            self.processingLog.append((text, type))
