import os
from struct import unpack


class ReadBlocketteError(Exception):
    def __init__(self, errno, msg):
        self.msg = msg


def readASCII(path, fileName, byteorder):
    """
    test function
    """
    file = open(os.path.join(path, fileName), 'rb')
    databytes = file.read()
    file.close()
    followingBlktsTotal = unpack('%s%s' % (byteorder, 'B'),
                                 databytes[39:40])[0]
    if followingBlktsTotal > 1:
        nextBlktByteNo = 48 + 8  # header + blkt1000(SEED info)
    else:
        nextBlktByteNo = 0
    logText = ""
    while nextBlktByteNo != 0:
        nextBlktByteNo, info = readNextBlkt(
            nextBlktByteNo, databytes, byteorder)
        logText += info

    print("INFO:\n", logText)


# ________________ based on mseedpeek.libtrace.Mseed.blkXXX________________
# https://docs.python.org/3/library/struct.html
# SEED Manual V2.4 - Chapter 8 - Data Record
def readNextBlkt(bNo, databytes, byteorder):
    """
    :param bNo: next blockette Byte Number
    :param databytes: file's data in byte
    :param byteorder: big/little endian
    :param key: (net, stat, loc)
    :param chanID: channel
    :return:
        nextBNo: byte number of next blockette
        info: info read from this blockette
    """
    blocketteType = unpack('%s%s' % (byteorder, 'H'),
                           databytes[bNo:bNo + 2])[0]
    nextBNo = unpack('%s%s' % (byteorder, 'H'),
                     databytes[bNo + 2:bNo + 4])[0]

    try:
        # readBlkt will skip first 4 bytes (HH) as they are already read
        info = eval("readBlkt%s(%s, %s, '%s')"
                    % (blocketteType, bNo, databytes, byteorder))
    except NameError:
        raise ReadBlocketteError(f"Function to read blockette {blocketteType} "
                                 f"isn't implemented yet.")
    return nextBNo, info


def readBlkt500(bNo, databytes, byteorder):
    logText = "\nVCO Correction: %s" % unpack(
        '%s%s' % (byteorder, 'f'), databytes[bNo + 4:bNo + 8])[0]
    t = {}
    (t['year'], t['doy'], t['hour'], t['min'], t['sec'],
     junk, t['micro']) = unpack(
        '%s%s' % (byteorder, 'HHBBBBH'), databytes[bNo + 8:bNo + 18])
    logText += ("\nTime of exception: %(year)s:%(doy)s:%(hour)s:%(min)s:"
                "%(sec)s:%(micro)s" % t)
    logText += "\nMicro sec: %s" % unpack(
        '%s%s' % (byteorder, 'B'), databytes[bNo + 18:bNo + 19])[0]
    logText += "\nReception Quality: %s" % unpack(
        '%s%s' % (byteorder, 'B'), databytes[bNo + 19:bNo + 20])[0]
    logText += "\nException Count: %s" % unpack(
        '%s%s' % (byteorder, 'I'), databytes[bNo + 20:bNo + 24])[0]
    logText += "\nException Type: %s" % unpack(
        '%s%s' % (byteorder, '16s'), databytes[bNo + 24:bNo + 40])[0].strip()
    logText += "\nClock Model: %s" % unpack(
        '%s%s' % (byteorder, '32s'), databytes[bNo + 40:bNo + 72])[0].strip()
    logText += "\nClock Status: %s" % unpack(
        '%s%s' % (byteorder, '128s'), databytes[bNo + 72:bNo + 200])[0].strip()
    return logText


def readBlkt2000(bNo, databytes, byteorder):
    blktLen = unpack(
        '%s%s' % (byteorder, 'H'), databytes[bNo + 4:bNo + 6])[0]
    logText = "\nTotal Blockette length: %s bytes" % blktLen
    logText += "\nOffset to Opaque Data: %s" % unpack(
        '%s%s' % (byteorder, 'H'), databytes[bNo + 6:bNo + 8])[0]
    logText += "\nRecord number: %s" % unpack(
        '%s%s' % (byteorder, 'I'), databytes[bNo + 8:bNo + 12])[0]
    logText += "\nData Word order: %s" % unpack(
        '%s%s' % (byteorder, 'B'), databytes[bNo + 12:bNo + 13])[0]
    logText += "\nOpaque Data flags: %s" % unpack(
        '%s%s' % (byteorder, 'B'), databytes[bNo + 13:bNo + 14])[0]
    opaqueHeaderTotal = unpack(
        '%s%s' % (byteorder, 'B'), databytes[bNo + 14:bNo + 15])[0]
    logText += "\nNumber of Opaque Header fields: %s" % opaqueHeaderTotal
    n = bNo + 15
    c = 0
    headerLen = 0
    for i in range(opaqueHeaderTotal):
        hfield = ''
        hchar = ''
        while hchar != '~':
            hfield += hchar
            hchar = unpack(
                '%s%s' % (byteorder, '1s'), databytes[n + c:n + c + 1])[
                0].decode()
            headerLen = c + 1
            c += 1
        logText += "\nOpaque Header %s: %s" % (i, hfield)

    opaqueDataLength = blktLen - 15 - headerLen
    logText += "\nOpaque Data: %s" % unpack(
        '%s%s' % (byteorder, '%ss' % opaqueDataLength),
        databytes[n + headerLen:n + headerLen + opaqueDataLength])

    return logText


if __name__ == '__main__':
    readASCII(
        "/Volumes/UNTITLED/fromCloud/qpeek/5244.sdr", "DT0001__.OCF", ">")
