from PySide2 import QtCore, QtGui, QtWidgets


class CalendarWidget(QtWidgets.QCalendarWidget):

    def __init__(self, parent):
        super().__init__(parent)
        self.setupUi()
        self._show_day_of_year = \
            getattr(self, 'toggle_day_of_year', None) is None

    def setupUi(self):
        self.setMinimumWidth(300)
        self.setMinimumHeight(275)

        nav_bar = self.findChild(
            QtWidgets.QWidget, 'qt_calendar_navigationbar'
        )

        if nav_bar:
            self.toggle_day_of_year = QtWidgets.QCheckBox('Show DOY')
            nav_bar.layout().insertWidget(2, self.toggle_day_of_year)

            self.toggle_day_of_year.toggled.connect(self.set_show_day_of_year)
            self.toggle_day_of_year.setCheckable(True)

            palette = self.toggle_day_of_year.palette()
            palette.setColor(QtGui.QPalette.WindowText, QtGui.QColor('white'))
            self.toggle_day_of_year.setPalette(palette)

            self.toggle_day_of_year.show()

    def show_day_of_year(self):
        return self._show_day_of_year

    def paintCell(self, painter, rect, date):
        super().paintCell(painter, rect, date)

        painter.save()

        if self.show_day_of_year():
            color = (QtGui.QColor('red') if date != self.selectedDate()
                     else QtGui.QColor('white'))
            font = painter.font()
            font.setPointSize(font.pointSize() * 0.9)
            painter.setFont(font)
            painter.setPen(color)
            painter.drawText(rect.bottomLeft(), str(date.dayOfYear()))
        painter.restore()

    @QtCore.Slot(bool)
    def set_show_day_of_year(self, value):
        self._show_day_of_year = value
        self.updateCells()
