"""
param_dialog.py
GUI to add/dit/remove params
NOTE: Cannot remove or change params that are already used for channels.
"""

from PySide2 import QtWidgets


from sohstationviewer.view.core.db_gui_superclass import Ui_DBInfoDialog
from sohstationviewer.view.core import plotting_widget
from sohstationviewer.database.proccessDB import executeDB

from sohstationviewer.conf.dbSettings import dbConf


class ParamDialog(Ui_DBInfoDialog):
    def __init__(self, parent):
        super().__init__(
            parent,
            ['No.', 'Param', 'Plot Type', 'ValueColors', 'Height    '],
            'param', 'parameters',
            resize_content_columns=[0, 3])
        self.setWindowTitle("Edit/Add/Delete Parameters")

    def add_row(self, row_idx, fk=False):
        self.addWidget(None, row_idx, 0)       # No.
        self.addWidget(self.data_list, row_idx, 1, foreign_key=fk)
        self.addWidget(self.data_list, row_idx, 2,
                       choices=[''] + sorted(plotting_widget.plotFunc.keys()))

        self.addWidget(self.data_list, row_idx, 3)
        self.addWidget(self.data_list, row_idx, 4, range=[0, 10])

    def get_data_list(self):
        param_rows = executeDB('SELECT * FROM Parameters')
        return [[d[0],
                 '' if d[1] is None else d[1],
                 d[2]]
                for d in param_rows]

    def get_row_inputs(self, row_idx):
        # check value_colors string
        value_colors_string = self.data_table_widget.cellWidget(
            row_idx, 3).text().strip()
        value_colors = value_colors_string.split("|")
        for vc in value_colors:
            if not dbConf['valColRE'].match(vc):
                msg = (f"The valueColor is requested for  '{vc}' at line "
                       f"{row_idx}does not match the required format:"
                       f"[+]value:color with color=R,Y,G,M,C."
                       f"\n Ex: 2.3:C|+4:M")
                QtWidgets.QMessageBox.information(self, "Error", msg)
        return [
            self.data_table_widget.cellWidget(row_idx, 1).text().strip(),
            self.data_table_widget.cellWidget(row_idx, 2).currentText(),
            self.data_table_widget.cellWidget(row_idx, 3).text().strip(),
            int(self.data_table_widget.cellWidget(row_idx, 4).text())
        ]

    def update_data(self, row, widget_idx, list_idx):
        insertsql = (f"INSERT INTO Parameters VALUES"
                     f"('{row[0]}', '{row[1]}', {row[2]})")
        updatesql = (f"UPDATE Parameters SET param='{row[0]}', "
                     f"plotType='{row[1]}', height={row[2]} "
                     f"WHERE param='%s'")
        return super().update_data(
            row, widget_idx, list_idx, insertsql, updatesql)
