import sys
import platform
import os
from PySide2 import QtWidgets
from functools import partial


"""
Dialog includes buttons.
Click on a button will close the dialog and the index of the clicked button
can be retrieved in obj.ret
"""


class SelectButtonDialog(QtWidgets.QDialog):
    def __init__(self, parent=None, message='',
                 buttonLabels=['test1', 'test2']):
        super(SelectButtonDialog, self).__init__()
        self.ret = -1
        height = 50 * (1 + len(buttonLabels) / 3)
        self.setGeometry(100, 100, 900, height)
        mainLayout = QtWidgets.QVBoxLayout()
        self.setLayout(mainLayout)

        label = QtWidgets.QLabel(message)
        mainLayout.addWidget(label, 0)

        buttonsLayout = QtWidgets.QGridLayout()
        mainLayout.addLayout(buttonsLayout)
        buttons = []
        r = -1
        for idx, label in enumerate(buttonLabels):
            c = idx % 3
            if c == 0:
                r += 1
            buttons.append(QtWidgets.QPushButton(label))
            buttons[idx].clicked.connect(partial(self.buttonClick, idx))
            buttonsLayout.addWidget(buttons[idx], r, c)

    def buttonClick(self, idx):
        self.ret = idx
        self.close()


if __name__ == '__main__':
    os_name, version, *_ = platform.platform().split('-')
    if os_name == 'macOS':
        os.environ['QT_MAC_WANTS_LAYER'] = '1'
    app = QtWidgets.QApplication(sys.argv)

    test = SelectButtonDialog(None, "Testing result from buttons",
                              ['test01', 'test02', 'test03',
                               'test11', 'test12', 'test13'])
    test.exec_()

    print("return Code:", test.ret)
    sys.exit(app.exec_())
