# UI and connectSignals for MainWindow

from PySide2 import QtCore, QtGui, QtWidgets

from sohstationviewer.view.core.calendar_widget import CalendarWidget

from sohstationviewer.view.core.plotting_widget import PlottingWidget
from sohstationviewer.conf import constants


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        self.MainWindow = MainWindow
        MainWindow.resize(1798, 1110)
        MainWindow.setWindowTitle("SOH Station Viewer")
        self.centralWidget = QtWidgets.QWidget(MainWindow)
        MainWindow.setCentralWidget(self.centralWidget)

        self.trackingInfoTextBrowser = QtWidgets.QTextBrowser(
            self.centralWidget)

        mainLayout = QtWidgets.QVBoxLayout()
        mainLayout.setContentsMargins(5, 5, 5, 5)
        mainLayout.setSpacing(0)
        self.centralWidget.setLayout(mainLayout)
        self.setFirstRow(mainLayout)
        self.setSecondRow(mainLayout)

        self.trackingInfoTextBrowser.setFixedHeight(60)
        mainLayout.addWidget(self.trackingInfoTextBrowser)
        self.createMenuBar(MainWindow)
        self.connectSignals(MainWindow)

    def setFirstRow(self, mainLayout):
        hLayout = QtWidgets.QHBoxLayout()
        hLayout.setContentsMargins(2, 2, 2, 2)
        hLayout.setSpacing(8)
        mainLayout.addLayout(hLayout)

        self.cwdButton = QtWidgets.QPushButton(
            "Main Data Directory", self.centralWidget)
        hLayout.addWidget(self.cwdButton)

        self.cwdLineEdit = QtWidgets.QLineEdit(
            self.centralWidget)
        hLayout.addWidget(self.cwdLineEdit, 1)

        hLayout.addSpacing(40)

        hLayout.addWidget(QtWidgets.QLabel('From'))
        self.timeFromDateEdit = QtWidgets.QDateEdit(
            self.centralWidget)
        self.timeFromDateEdit.setCalendarPopup(True)
        self.timeFromDateEdit.setDisplayFormat("yyyy-MM-dd")
        hLayout.addWidget(self.timeFromDateEdit)

        hLayout.addWidget(QtWidgets.QLabel('To'))
        self.timeToDateEdit = QtWidgets.QDateEdit(
            self.centralWidget)
        self.timeToDateEdit.setCalendarPopup(True)
        self.timeToDateEdit.setDisplayFormat("yyyy-MM-dd")
        hLayout.addWidget(self.timeToDateEdit)

    def setSecondRow(self, mainLayout):
        hLayout = QtWidgets.QHBoxLayout()
        hLayout.setContentsMargins(0, 0, 0, 0)

        mainLayout.addLayout(hLayout)

        self.setControlColumn(hLayout)

        self.plottingWidget = PlottingWidget(self.MainWindow,
                                             self.trackingInfoTextBrowser,
                                             'SOHWidget')

        hLayout.addWidget(self.plottingWidget, 2)

    def setControlColumn(self, parentLayout):
        # TODO: search for sideScrollArea
        leftWidget = QtWidgets.QWidget(self.centralWidget)
        leftWidget.setFixedWidth(240)
        leftWidget.setMinimumHeight(650)
        parentLayout.addWidget(leftWidget)

        leftLayout = QtWidgets.QVBoxLayout()
        leftLayout.setContentsMargins(0, 0, 0, 0)
        leftLayout.setSpacing(0)
        leftWidget.setLayout(leftLayout)

        self.openFilesList = QtWidgets.QListWidget(
            self.centralWidget)
        leftLayout.addWidget(self.openFilesList, 1)
        pal = self.openFilesList.palette()
        pal.setColor(QtGui.QPalette.Highlight, QtGui.QColor(128, 255, 128))
        pal.setColor(QtGui.QPalette.HighlightedText, QtGui.QColor(0, 0, 0))
        self.openFilesList.setPalette(pal)
        # allow multiple-line selection
        self.openFilesList.setSelectionMode(
            QtWidgets.QAbstractItemView.ExtendedSelection)

        self.insideDirCheckBox = QtWidgets.QCheckBox(
            'Inside Directory', self.centralWidget)
        leftLayout.addWidget(self.insideDirCheckBox)

        searchGrid = QtWidgets.QGridLayout()
        # searchGrid.setContentsMargins(0, 0, 0, 0)
        leftLayout.addLayout(searchGrid)

        self.searchLineEdit = QtWidgets.QLineEdit(self.centralWidget)
        self.searchLineEdit.setPlaceholderText('Search...')
        searchGrid.addWidget(self.searchLineEdit, 0, 0, 1, 3)

        self.clearButton = QtWidgets.QPushButton('Clear', self.centralWidget)
        self.clearButton.setFixedWidth(65)
        searchGrid.addWidget(self.clearButton, 0, 3, 1, 1)

        searchGrid.addWidget(QtWidgets.QLabel('List:'), 1, 0, 1, 1)
        self.fileListLogCheckBox = QtWidgets.QCheckBox(
            '.log', self.centralWidget)
        searchGrid.addWidget(self.fileListLogCheckBox, 1, 1, 1, 1)
        self.fileListZipCheckBox = QtWidgets.QCheckBox(
            '.zip', self.centralWidget)
        searchGrid.addWidget(self.fileListZipCheckBox, 1, 2, 1, 1)

        self.replotButton = QtWidgets.QPushButton('RePlot', self.centralWidget)

        self.replotButton.setFixedWidth(95)

        searchGrid.addWidget(self.replotButton, 1, 3, 1, 1)

        backgroundLayout = QtWidgets.QHBoxLayout()
        # backgroundLayout.setContentsMargins(0, 0, 0, 0)
        leftLayout.addLayout(backgroundLayout)
        backgroundLayout.addWidget(QtWidgets.QLabel('Background:     '))
        self.backgroundBlackRadioButton = QtWidgets.QRadioButton(
            'B', self.centralWidget)
        backgroundLayout.addWidget(self.backgroundBlackRadioButton)
        self.backgroundWhiteRadioButton = QtWidgets.QRadioButton(
            'W', self.centralWidget)
        backgroundLayout.addWidget(self.backgroundWhiteRadioButton)

        self.addSeperationLine(leftLayout)

        gapLayout = QtWidgets.QHBoxLayout()
        leftLayout.addLayout(gapLayout)

        self.detectGapCheckBox = QtWidgets.QCheckBox(
            'DetectGap   Len:', self.centralWidget)
        gapLayout.addWidget(self.detectGapCheckBox)

        self.gapLenLineEdit = QtWidgets.QLineEdit(self.centralWidget)
        gapLayout.addWidget(self.gapLenLineEdit)

        gapLayout.addWidget(QtWidgets.QLabel('m'))

        massPosLayout = QtWidgets.QHBoxLayout()
        # massPosLayout.setContentsMargins(0, 0, 0, 0)
        leftLayout.addLayout(massPosLayout)
        massPosLayout.addWidget(QtWidgets.QLabel('Mass Pos:'))
        self.massPos123CheckBox = QtWidgets.QCheckBox(
            '123', self.centralWidget)
        massPosLayout.addWidget(self.massPos123CheckBox)
        self.massPos456CheckBox = QtWidgets.QCheckBox(
            '456', self.centralWidget)
        massPosLayout.addWidget(self.massPos456CheckBox)

        self.addSeperationLine(leftLayout)

        dsGrid = QtWidgets.QGridLayout()
        # dsGrid.setContentsMargins(0, 0, 0, 0)
        leftLayout.addLayout(dsGrid)

        dsGrid.addWidget(QtWidgets.QLabel('DSs:'), 0, 0, 2, 1,
                         QtGui.Qt.AlignVCenter)
        self.dsCheckBoxes = []
        count = 0
        for r in range(2):
            for c in range(4):
                count += 1
                self.dsCheckBoxes.append(
                    QtWidgets.QCheckBox('%s' % count, self.centralWidget))
                dsGrid.addWidget(self.dsCheckBoxes[count - 1], r, c + 1)

        wfGrid = QtWidgets.QGridLayout()
        leftLayout.addLayout(wfGrid)
        self.wfAllCheckBox = QtWidgets.QCheckBox(
            'All  WFChans:', self.centralWidget)
        wfGrid.addWidget(self.wfAllCheckBox, 0, 0, 1, 2)

        self.wfChansLineEdit = QtWidgets.QLineEdit(self.centralWidget)
        wfGrid.addWidget(self.wfChansLineEdit, 0, 3, 1, 2)

        self.tpsCheckBox = QtWidgets.QCheckBox('TPS', self.centralWidget)
        wfGrid.addWidget(self.tpsCheckBox, 1, 0)
        self.addSeperationLine(leftLayout)

        chanLayout = QtWidgets.QHBoxLayout()
        chanLayout.setContentsMargins(0, 0, 0, 0)
        chanLayout.setSpacing(0)
        leftLayout.addLayout(chanLayout)

        self.allChanCheckBox = QtWidgets.QCheckBox(
            'AllChan  ', self.centralWidget)
        self.allChanCheckBox.setChecked(True)
        chanLayout.addWidget(self.allChanCheckBox)

        self.preferChanButton = QtWidgets.QPushButton(
            'Pref', self.centralWidget)
        self.preferChanButton.setFixedWidth(47)
        chanLayout.addWidget(self.preferChanButton)

        chanLayout.addWidget(QtWidgets.QLabel('Cur'))
        self.currIDsNameLineEdit = QtWidgets.QLineEdit(self.centralWidget)
        chanLayout.addWidget(self.currIDsNameLineEdit)

        submitLayout = QtWidgets.QHBoxLayout()
        # submitLayout.setContentsMargins(5, 0, 0, 5)
        submitLayout.setSpacing(5)
        leftLayout.addLayout(submitLayout)
        self.readButton = QtWidgets.QPushButton('Read', self.centralWidget)
        submitLayout.addWidget(self.readButton)
        self.stopButton = QtWidgets.QPushButton('Stop', self.centralWidget)
        submitLayout.addWidget(self.stopButton)
        self.writePSButton = QtWidgets.QPushButton(
            'Write .ps', self.centralWidget)
        submitLayout.addWidget(self.writePSButton)

        self.infoListWidget = QtWidgets.QListWidget(self.centralWidget)
        leftLayout.addWidget(self.infoListWidget, 1)

    def createMenuBar(self, MainWindow):
        mainMenu = MainWindow.menuBar()
        fileMenu = mainMenu.addMenu("File")
        commandMenu = mainMenu.addMenu("Commands")
        optionMenu = mainMenu.addMenu("Options")
        self.formMenu = mainMenu.addMenu("Forms")
        databaseMenu = mainMenu.addMenu("Database")
        helpMenu = mainMenu.addMenu("Help")

        # exitAction = QtWidgets.QAction(QtGui.QIcon('exit.png'), "Exit", self)
        # exitAction.setShortcut("Ctrl+X")
        self.createFileMenu(MainWindow, fileMenu)
        self.createCommandMenu(MainWindow, commandMenu)
        self.createOptionMenu(MainWindow, optionMenu)
        self.createDatabaseMenu(MainWindow, databaseMenu)
        self.createHelpMenu(MainWindow, helpMenu)

    def createFileMenu(self, MainWindow, menu):
        self.exitAction = QtWidgets.QAction('Close', MainWindow)
        menu.addAction(self.exitAction)

    def createCommandMenu(self, MainWindow, menu):
        self.GPSPlotterAction = QtWidgets.QAction(
            'GPS Plotter', MainWindow)
        menu.addAction(self.GPSPlotterAction)

        self.logSearchAction = QtWidgets.QAction(
            'Log Search', MainWindow)
        menu.addAction(self.logSearchAction)

        self.plotTimeRangesAction = QtWidgets.QAction(
            'Plot Time Ranges', MainWindow)
        menu.addAction(self.plotTimeRangesAction)

        self.plotPositionAction = QtWidgets.QAction(
            'Log Search', MainWindow)
        menu.addAction(self.plotPositionAction)

        exportTTTimesAsMenu = QtWidgets.QMenu(
            'Export TT Times As:', MainWindow)
        menu.addMenu(exportTTTimesAsMenu)

        self.exportDeployFileLineAction = QtWidgets.QAction(
            'Deployment File (Line)', MainWindow)
        exportTTTimesAsMenu.addAction(self.exportDeployFileLineAction)

        self.exportDeployFileBlockAction = QtWidgets.QAction(
            'Deployment File (Block)', MainWindow)
        exportTTTimesAsMenu.addAction(self.exportDeployFileBlockAction)

        self.exportTPSShotFileAction = QtWidgets.QAction(
            'TPS Shotfile/Geometry', MainWindow)
        exportTTTimesAsMenu.addAction(self.exportTPSShotFileAction)

        self.exportShotInfoAction = QtWidgets.QAction(
            'Shot Info', MainWindow)
        exportTTTimesAsMenu.addAction(self.exportShotInfoAction)

    def createOptionMenu(self, MainWindow, menu):
        self.plotTimingProblemsAction = QtWidgets.QAction(
            'Plot Timing Problems', MainWindow)
        self.plotTimingProblemsAction.setCheckable(True)
        menu.addAction(self.plotTimingProblemsAction)

        self.filterNonSOHAction = QtWidgets.QAction(
            'Filter Out Non-SOH Lines', MainWindow)
        self.filterNonSOHAction.setCheckable(True)
        menu.addAction(self.filterNonSOHAction)

        menu.addSeparator()

        Q330GainMenu = QtWidgets.QMenu('Q330 Gain:', MainWindow)
        menu.addMenu(Q330GainMenu)
        self.Q330LowGainAction = QtWidgets.QAction('Low', MainWindow)
        self.Q330LowGainAction.setCheckable(True)
        Q330GainMenu.addAction(self.Q330LowGainAction)
        self.Q330HighGainAction = QtWidgets.QAction('High', MainWindow)
        self.Q330HighGainAction.setCheckable(True)
        Q330GainMenu.addAction(self.Q330HighGainAction)

        self.addMPToSOHAction = QtWidgets.QAction(
            'Add Mass Positions to SOH Messages', MainWindow)
        self.addMPToSOHAction.setCheckable(True)
        menu.addAction(self.addMPToSOHAction)

        MPColoringMenu = QtWidgets.QMenu('MP Coloring:', MainWindow)
        menu.addMenu(MPColoringMenu)
        self.MPRegularColorAction = QtWidgets.QAction(
            '0.5, 2.0, 4.0, 7.0 (Regular)', MainWindow)
        self.MPRegularColorAction.setCheckable(True)
        MPColoringMenu.addAction(self.MPRegularColorAction)
        self.MPTrilliumColorAction = QtWidgets.QAction(
            '0.5, 1.8, 2.4, 3.5 (Trillium)', MainWindow)
        self.MPTrilliumColorAction.setCheckable(True)
        MPColoringMenu.addAction(self.MPTrilliumColorAction)

        menu.addSeparator()

        self.addPositionsToETAction = QtWidgets.QAction(
            'Add Positions to ET Lines', MainWindow)
        self.addPositionsToETAction.setCheckable(True)
        menu.addAction(self.addPositionsToETAction)

        self.readAntLogAction = QtWidgets.QAction(
            'Read Antelope-Produced Log Files', MainWindow)
        self.readAntLogAction.setCheckable(True)
        menu.addAction(self.readAntLogAction)

        menu.addSeparator()

        self.calcFileSizesAction = QtWidgets.QAction(
            'Calculate File Sizes', MainWindow)
        menu.addAction(self.calcFileSizesAction)
        self.warnIfBigAction = QtWidgets.QAction(
            'Warn If Big', MainWindow)
        menu.addAction(self.warnIfBigAction)

        menu.addSeparator()

        dateFormatMenu = QtWidgets.QMenu('Date Format:', MainWindow)
        menu.addMenu(dateFormatMenu)
        self.YYYYDOYAction = QtWidgets.QAction(
            'YYYY:DOY', MainWindow)
        self.YYYYDOYAction.setCheckable(True)
        dateFormatMenu.addAction(self.YYYYDOYAction)
        self.YYYY_MM_DDAction = QtWidgets.QAction(
            'YYYY-MM-DD', MainWindow)
        self.YYYY_MM_DDAction.setCheckable(True)
        dateFormatMenu.addAction(self.YYYY_MM_DDAction)
        self.YYYYMMMDDAction = QtWidgets.QAction(
            'YYYYMMMDD', MainWindow)
        self.YYYYMMMDDAction.setCheckable(True)
        dateFormatMenu.addAction(self.YYYYMMMDDAction)

        menu.addSeparator()

        self.setFontSizeAction = QtWidgets.QAction(
            'Set Font Sizes', MainWindow)
        menu.addAction(self.setFontSizeAction)

    def createDatabaseMenu(self, MainWindow, menu):
        self.addEditDataTypeAction = QtWidgets.QAction(
            'Add/Edit Data Types', MainWindow)
        menu.addAction(self.addEditDataTypeAction)

        self.addEditParamAction = QtWidgets.QAction(
            'Add/Edit Parameters', MainWindow)
        menu.addAction(self.addEditParamAction)

        self.addEditChannelAction = QtWidgets.QAction(
            'Add/Edit Channels', MainWindow)
        menu.addAction(self.addEditChannelAction)

        self.viewPlotTypeAction = QtWidgets.QAction(
            'View Plot Types', MainWindow)
        menu.addAction(self.viewPlotTypeAction)

    def createHelpMenu(self, MainWindow, menu):
        self.calendarAction = QtWidgets.QAction(
            'Calendar', MainWindow)
        menu.addAction(self.calendarAction)

        self.aboutAction = QtWidgets.QAction(
            'About', MainWindow)
        menu.addAction(self.aboutAction)

    def addSeperationLine(self, layout):
        label = QtWidgets.QLabel()
        label.setFrameStyle(QtWidgets.QFrame.HLine | QtWidgets.QFrame.Sunken)
        label.setLineWidth(1)
        layout.addWidget(label)

    def resizeEvent(self, event):
        self.plottingWidget.init_size()

    def connectSignals(self, MainWindow):
        self.connectMenuSignals(MainWindow)
        self.connectWidgetSignals(MainWindow)

    def connectMenuSignals(self, MainWindow):
        # File
        self.exitAction.triggered.connect(MainWindow.close)

        # Commands

        # Options
        self.MPRegularColorAction.triggered.connect(
            lambda: setattr(MainWindow, 'massPosVoltRangeOpt', 'regular'))
        self.MPTrilliumColorAction.triggered.connect(
            lambda: setattr(MainWindow, 'massPosVoltRangeOpt', 'trillium'))

        self.YYYY_MM_DDAction.triggered.connect(
            lambda: self.setDateFormat('yyyy-MM-dd'))
        self.YYYYMMMDDAction.triggered.connect(
            lambda: self.setDateFormat('yyyyMMMdd'))
        self.YYYYDOYAction.triggered.connect(
            lambda: self.setDateFormat('yyyyDOY'))

        # Database
        self.addEditDataTypeAction.triggered.connect(
            MainWindow.openDataType)
        self.addEditParamAction.triggered.connect(
            MainWindow.openParam)
        self.addEditChannelAction.triggered.connect(
            MainWindow.openChannel)
        self.viewPlotTypeAction.triggered.connect(
            MainWindow.openPlotType)

        # Form

        # Help
        self.calendarAction.triggered.connect(MainWindow.openCalendarWidget)

    def connectWidgetSignals(self, MainWindow):
        MainWindow.currentDirectoryChanged.connect(self.cwdLineEdit.setText)
        # first Row
        self.cwdButton.clicked.connect(MainWindow.changeCurrentDirectory)
        self.timeToDateEdit.setCalendarWidget(CalendarWidget(MainWindow))
        self.timeToDateEdit.setDate(QtCore.QDate.currentDate())

        self.timeFromDateEdit.setCalendarWidget(CalendarWidget(MainWindow))
        self.timeFromDateEdit.setDate(QtCore.QDate.fromString(
            constants.DEFAULT_START_TIME, QtCore.Qt.ISODate
        ))

        # second Row
        self.openFilesList.itemDoubleClicked.connect(
            MainWindow.openFilesListItemDoubleClicked)

        # self.resetButton.clicked.connect(MainWindow.resetView)

        self.replotButton.clicked.connect(MainWindow.replotLoadedData)

        self.allChanCheckBox.clicked.connect(
            MainWindow.allChanClicked)
        self.preferChanButton.clicked.connect(
            MainWindow.openChannelPreferences)
        self.readButton.clicked.connect(MainWindow.readSelectedFiles)
